#!/usr/bin/python
# -*- coding: utf-8 -*-

'''

		Media Manager

		07/07/2021 08:40

		lvardon@laposte.net - 2021

		Licence libre

'''

import time, platform, os, sys
from pathlib import Path

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

import lib.setting as AppSetting
from configparser import ConfigParser

from libwidget.mediastype.mediatypehttp import MediaTypeHttp
from libwidget.mediastype.mediatypeaudio import MediaTypeAudio
from libwidget.mediastype.mediatypepdf import MediaTypePdf
from libwidget.mediastype.mediatypevideo import MediaTypeVideo
from libwidget.mediastype.mediatypephoto import MediaTypePhoto

from PyQt5 import QtWidgets
from PyQt5.Qt import QStandardItem
from PyQt5.QtWidgets import QMainWindow, QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox,QTableView, QHeaderView, QToolBar, QFileDialog
from PyQt5.QtWidgets import QDialog, QColorDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QBoxLayout, QScrollArea, QGraphicsRectItem, QGraphicsScene, QGraphicsItem, QGraphicsLineItem
from PyQt5.QtWidgets import QTableWidget, QTableWidgetItem, QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QRadioButton, QDateTimeEdit, QToolButton, QSplitter,QFrame
from PyQt5.QtGui import QCursor, QFont, QIcon, QImage, QPixmap, QBrush, QColor, QTransform, QPen, QPainterPath, QPainter
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot, QSettings, QModelIndex, QDateTime, QRect, QPoint, QRectF, QPointF, QLineF

class MediaViewer(QtWidgets.QGraphicsView):

	photoClicked = pyqtSignal(QPoint)
	drawMode = False
	debugPrint = True

	def __init__(self, parent):
		super(MediaViewer, self).__init__(parent)
		self._zoom = 0
		self._empty = True
		self.parent = parent

		self._photo = QtWidgets.QGraphicsPixmapItem()

		self._scene = QGraphicsScene(self)		
		self.setScene(self._scene)

		self.setTransformationAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
		self.setResizeAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
		self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
		self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
		self.setBackgroundBrush(QBrush(QColor(100, 100, 100)))
		self.setFrameShape(QtWidgets.QFrame.NoFrame)

	def printDebug( self, *args ) :

		if self.debugPrint :
			message=""
			for messageElem in args:
				message = message + str(messageElem)			

			import inspect
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			AppLog.logContext = inspect.getframeinfo(frame)	

			AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)

	def setViewParams( self, viewParams ) :
		self.viewParams = viewParams

	def setPhotoFromPixmap(self, pixmap ) :

		self._zoom = 0
		self.pixmap = pixmap
        
		if self.pixmap and not self.pixmap.isNull():
			self._empty = False
			self.setDragMode(QtWidgets.QGraphicsView.ScrollHandDrag)
						
			self._photo.setPixmap(self.pixmap)
			sceneRect = QRectF( 0,0,pixmap.width(),pixmap.height())
			self._scene.setSceneRect(sceneRect)							
			self._scene.addItem(self._photo)
			
			self.printDebug("SetPhoto, Pixmap : ", pixmap.width(), ' x ' , pixmap.height())
			self.printDebug("SetPhoto, SceneRect : ", sceneRect.width(), ' x ' , sceneRect.height())		
			self.printDebug("SetPhoto, View : " , self.frameSize().width()," x ", self.frameSize().height() )
					
		else:
			self._empty = True
			self.setDragMode(QtWidgets.QGraphicsView.NoDrag)
			self._photo.setPixmap(QPixmap())

	def hasPhoto(self):
		return not self._empty
		
	def wheelEvent(self, event):
		if self.hasPhoto():

			if event.angleDelta().y() > 0:
				if self._zoom <= self.viewParams['zoomMax'] :
					factor = 1.25
					self._zoom += 1
					self.scale(factor, factor)
			else:
				if self._zoom >= self.viewParams['zoomMin'] :
					factor = 0.8
					self._zoom -= 1
					self.scale(factor, factor)

		self.printDebug("Zoom ", self._zoom)

	def setZoomZero(self) :
		
		self.resetTransform();()
		self._zoom  = 0	
		self.printDebug("Zoom zero, zoom=", self._zoom)

	def setZoomFit(self) :
		self.fitInView(self._scene.sceneRect(), Qt.KeepAspectRatio)
		self.printDebug("Zoom fit, zoom=", self._zoom)


class MediaManager(QDialog):

	debugPrint = True

	iniConfig = "config/mediamanager.ini"
	iconPath = "img/mediamanager/"
	
	# App config
	fileExplorer = "nemo"
	tempdir = "temp"
	mediaTypesVideoList = ["mp4", "xvid"]
	mediaTypesAudioList = ["flac", "mp3", "ogg"]
	mediaTypesPhotoList = ["png", "jpg", "tiff"]
	mediaTypesDocList = ["pdf"]

	parent = None
	isModified = False

	Reset = False
	Accept = None
	currentItem = None	

	filename = ""
	filenamelist = []
	filenamelistIndex = None
	
	rootMedia = ""

	viewParams = {

		'zoomMin' : -16,
		'zoomMax' : 16
	}
	
	log = None
	
	def __init__(self,  parent = None, filenamelist=None, rootMedia=None ):
		super(MediaManager, self).__init__(parent)
		
		self.rootMedia = rootMedia
		self.printDebug( "rootMedia = " + str(self.rootMedia))		

		self.loadSettings()
		
		self.parent = parent
		self.setupUi()

		self.filenamelistIndex = 0
		self.filenamelist = filenamelist
		if filenamelist is not None :
			self.filename = filenamelist[self.filenamelistIndex]
		else :
			self.filename = None

	def setupUi( self) :

		layout = QVBoxLayout(self)

		toolBar = QToolBar()
		layout.addWidget(toolBar)

		self.buttonSelectFile = QToolButton()
		self.buttonSelectFile.clicked.connect(self.onSelectFile)
		self.buttonSelectFile.setIcon(QIcon(QPixmap(self.iconPath + "media-add.png")))
		self.buttonSelectFile.setToolTip('Ajouter un média')
		toolBar.addWidget(self.buttonSelectFile)

		self.buttonEditFile = QToolButton()
		self.buttonEditFile.clicked.connect(self.onEditFile)
		self.buttonEditFile.setIcon(QIcon(QPixmap(self.iconPath + "media-edit.png")))
		self.buttonEditFile.setToolTip('Editer ce média')
		toolBar.addWidget(self.buttonEditFile)		
		
		self.mediaName = QLineEdit()
		self.mediaName.textChanged.connect(self.onMediaNameEditChanged)
		self.mediaNameValid = QToolButton()
		self.mediaNameValid.setIcon(QIcon(QPixmap(self.iconPath + "media-rename.png")))
		self.mediaNameValid.clicked.connect(self.onMediaNameEditValid)
		self.mediaNameValid.setToolTip("Valider le changement de nom du média")
		toolBar.addWidget(self.mediaName)
		toolBar.addWidget(self.mediaNameValid)

		self.setGeometry(80, 300, 800, 640)

		self.viewer = MediaViewer(self)
		
		self.mediaInfoText = QTableWidget(0,2)
		
		self.mediaInfoText.setHorizontalHeaderLabels(['Propriété', 'Valeur'])
		
		self.mediaInfoText.horizontalHeader().setStretchLastSection(True)		
		# FIX
		#self.mediaInfoText.horizontalHeader().setSectionResizeMode(  QHeaderView.Stretch )
		
		self.displayMediaMetaDatas( {} ) 
		
		self.rightSplitter = QSplitter(Qt.Vertical)
		self.rightSplitter.addWidget(self.mediaInfoText)		
		
		self.mainSplitter = QSplitter(Qt.Horizontal)
		
		self.mainSplitter.addWidget(self.viewer)
		self.mainSplitter.addWidget(self.rightSplitter)

		layout.addWidget(self.mainSplitter)
		
		# OK and Cancel buttons
		buttons = QDialogButtonBox(
			QDialogButtonBox.Reset | QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
			Qt.Horizontal, self)
		buttons.accepted.connect(self.onAccept)
		buttons.rejected.connect(self.onReject)
		buttons.button(QDialogButtonBox.Reset).clicked.connect(self.onReset)
		buttons.button(QDialogButtonBox.Ok).setToolTip('Valider')
		buttons.button(QDialogButtonBox.Cancel).setToolTip('Annuler')
		buttons.button(QDialogButtonBox.Reset).setToolTip('Effacer la valeur')

		layout.addWidget(buttons)

		self.desktop = QApplication.desktop()
		self.screenRect = self.desktop.screenGeometry()
		self.screenHeight = self.screenRect.height()
		self.screenWidth = self.screenRect.width()		
		self.printDebug( "screen size: ", self.screenWidth, " x ", self.screenHeight )

		self.viewer.setViewParams( self.viewParams )

	def printDebug( self, *args ) :

		if self.debugPrint :
			message=""
			for messageElem in args:
				message = message + str(messageElem)			

			import inspect
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			AppLog.logContext = inspect.getframeinfo(frame)	

			AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)

	def showEvent( self, event) :

		self.printDebug('showEvent')

		self.mainSplitter.setSizes([100,5])
		self.rightSplitter.setSizes([1,75])

		self.loadWindowState()

		self.viewer.setZoomFit()

		self.setWindowTitle("Selecteur de média")

	def onToFileCheck( self ) :

		if self.isModified is True :

			self.onMediaNameEditValid()

			ret = QMessageBox.question(self, 'Confirmez', "Sauvegarder avant de quitter ?", QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Cancel)

			if ret == QMessageBox.Yes:
				self.printDebug('Button QMessageBox.Yes clicked.')
				self.onToFile()
				return True

			if ret == QMessageBox.No:
				return True

			if ret == QMessageBox.Cancel:
				return False

		return True

	def setModified (self, modifiedState) :
		self.isModified  = modifiedState		


	def onEditFile(self) :
		
		# Remove "rootmedia" from url
		if self.filename.find(self.rootMedia) == 0 :										
			filenameEdit = self.filename.replace( self.rootMedia, '')
		
		from libwidget.photomanager import PhotoManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myPhotoSelector = PhotoManager( rootMedia = currentProject.rootMedia )		
		myPhotoSelector.loadDataList( filenameEdit.split(',') )		
		myPhotoSelector.disableButtons()
		result = myPhotoSelector.exec_()
		filenamelist = myPhotoSelector.getValue()
		if filenamelist is not None :
			self.printDebug("Edited filenamelist = " , filenamelist)


	def onSelectFile(self) :

		if self.onToFileCheck() is False :
			return
		
		defaultDir = self.rootMedia

		filedialog = QFileDialog(self,
									caption = "Selectionner le nom du média à ouvrir",
									directory = defaultDir
									)

		mediasPhotoExt = ' *.' + ' *.'.join(self.mediaTypesPhotoList)
		mediasVideoExt = ' *.' + ' *.'.join(self.mediaTypesVideoList)
		mediasAudioExt = ' *.' + ' *.'.join(self.mediaTypesAudioList)
		mediasDocExt = ' *.' + ' *.'.join(self.mediaTypesDocList)	
		mediaAllExt = mediasPhotoExt + ' '+ mediasVideoExt + ' ' + mediasAudioExt + ' ' + mediasDocExt

		self.printDebug( "media ALL extention list : " + str(mediaAllExt))
		self.printDebug( "media Photo extention list : " + str(mediasPhotoExt))	
		self.printDebug( "media Vidéo extention list : " + str(mediasVideoExt))	
		self.printDebug( "media Document extention list : " + str(mediasDocExt))	
		self.printDebug( "media Aufio extention list : " + str(mediasAudioExt))	

		filedialog.setNameFilter("Tous les médias ("+mediaAllExt+");;Image ("+mediasPhotoExt+");;Vidéo ("+mediasVideoExt+");;Audio ("+mediasAudioExt+");;Document ("+mediasDocExt+");;Tous (*.*)")
		selected = filedialog.exec()
		if selected:

			if self.filenamelist[0] !="" and self.filename != "" and len(self.filenamelist) > 0 :

				self.printDebug( "Insert REPLACE , pos : " , self.filenamelistIndex )
				self.filenamelist[self.filenamelistIndex] = filedialog.selectedFiles()[0]
				self.filename  = filedialog.selectedFiles()[0]

			else :
				self.filenamelistIndex = 0
				self.filenamelist = [""]
				self.printDebug( "Insert REPLACE , pos : " , self.filenamelistIndex )
				self.filenamelist[self.filenamelistIndex] = filedialog.selectedFiles()[0]
				self.filename  = filedialog.selectedFiles()[0]

			self.loadFileData( self.filename )

		else:
			return

	def onAccept(self) :

		if self.onToFileCheck() is False :
			return

		self.printDebug("onAccept")
		self.Accept = True		
		self.accept()

	def onReject(self) :

		if self.onToFileCheck() is False :
			return

		self.printDebug("onReject")
		self.Accept = False
		self.reject()

	def onReset( self ) :

		if self.onToFileCheck() is False :
			return

		self.Reset = True
		self.accept()

	def contextMenuEvent(self, event):

		cmenu = QMenu(self)

		storeWindowStateAct = cmenu.addAction("Enregistrer la disposition")  # zoom, rotate, splitter
		storeWindowStateAct.triggered.connect(self.onStoreWindowState)

		zoomZeroAct = cmenu.addAction("Zoom zero")  
		zoomZeroAct.triggered.connect(self.onSetZoomZero)

		zoomFitAct = cmenu.addAction("Zoom idéal")  
		zoomFitAct.triggered.connect(self.onSetZoomFit)

		cmenu.addSeparator()

		copyPhotoNameAct = cmenu.addAction("Copier le nom du média")
		copyPhotoNameAct.triggered.connect(self.onCopyPhotoName)

		cmenu.addSeparator()
		
		opendirFileAct = cmenu.addAction("Ouvrir le dossier du média ...")
		opendirFileAct.triggered.connect(self.onOpendirFile)

		startFileAct = cmenu.addAction("Ouvrir le média ...")
		startFileAct.triggered.connect(self.onStartFile)
			
		action = cmenu.exec_(self.mapToGlobal(event.pos()))

	def loadSettings(self) :

		settings=QSettings(self.iniConfig, QSettings.IniFormat)

		self.tempdir = "temp"
		if settings.value('mediamanager/mediaTempDir') is not None:			
			self.tempdir = settings.value('mediamanager/mediaTempDir')						
		self.printDebug( "tempdir = " + str(self.tempdir))

		self.iconPath = "img/mediamanager/"
		if settings.value('mediaIconPath/iconPath') is not None:			
			self.iconPath = settings.value('mediaIconPath/iconPath')						
		self.printDebug( "iconPath = " + str(self.iconPath))

		self.fileExplorer = "nemo"

		if platform.system() == 'Darwin':       # macOS
			if settings.value('fileExplorerTux') is not None:			
				self.fileExplorer = settings.value('fileExplorerTux')				
		elif platform.system() == 'Windows':    # Windows
			if settings.value('fileExplorerWin') is not None:			
				self.fileExplorer = settings.value('fileExplorerWin')				
		else: # linux variants
			if settings.value('fileExplorerTux') is not None:			
				self.fileExplorer = settings.value('fileExplorerTux')						

		self.printDebug( "fileExplorer = " + str(self.fileExplorer))

		self.mediaTypesVideoList = "mp4,xvid"
		if settings.value('mediamanager/mediaTypesVideoList') is not None:			
			self.mediaTypesVideoList = settings.value('mediamanager/mediaTypesVideoList')						
		self.printDebug( "mediaTypesVideoList = " + str(self.mediaTypesVideoList))

		self.mediaTypesAudioList = "flac,mp3,ogg"
		if settings.value('mediamanager/mediaTypesAudioList') is not None:			
			self.mediaTypesAudioList = settings.value('mediamanager/mediaTypesAudioList')						
		self.printDebug( "mediaTypesAudioList = " + str(self.mediaTypesAudioList))

		self.mediaTypesPhotoList = "png, jpg, tiff"
		if settings.value('mediamanager/mediaTypesPhotoList') is not None:			
			self.mediaTypesPhotoList = settings.value('mediamanager/mediaTypesPhotoList')						
		self.printDebug( "mediaTypesPhotoList = " + str(self.mediaTypesPhotoList))

		self.mediaTypesDocList = "pdf"
		if settings.value('mediamanager/mediaTypesDocList') is not None:			
			self.mediaTypesDocList = settings.value('mediamanager/mediaTypesDocList')						
		self.printDebug( "mediaTypesDocList = " + str(self.mediaTypesDocList))

			
	def loadWindowState(self) :

		settings=QSettings(self.iniConfig, QSettings.IniFormat)

		if settings.value("geometry") is not None:
			self.restoreGeometry(settings.value("geometry", ""))

		try :
			rightSplitterSetting=settings.value("rightSplitterSetting")
			self.rightSplitter.restoreState(rightSplitterSetting)
		except Exception as e:
			self.printDebug(e)

		try :
			mainSplitterSetting=settings.value("mainSplitterSetting") #.toByteArray()
			self.mainSplitter.restoreState(mainSplitterSetting)
		except Exception as e:
			self.printDebug(e)

	def onStoreWindowState(self) :

		settings=QSettings(self.iniConfig, QSettings.IniFormat)

		settings.setValue("geometry", self.saveGeometry())

		mainSplitterSetting = self.mainSplitter.saveState()
		if mainSplitterSetting:
			settings.setValue("mainSplitterSetting", self.mainSplitter.saveState())

		rightSplitterSetting = self.rightSplitter.saveState()
		if rightSplitterSetting:
			settings.setValue("rightSplitterSetting", self.rightSplitter.saveState())

	def onSetZoomZero(self) :
		self.viewer.setZoomZero()

	def onSetZoomFit(self) :
		self.viewer.setZoomFit()

	def onStartFile( self ):

		p = str(Path(self.filename).resolve())

		AppLog.log( LOG_INFO , "Ouverture du fichier : " + p)					
		
		import subprocess, platform
		if platform.system() == 'Darwin':       # macOS
			AppLog.log( LOG_INFO , "Ouverture ...")									
			subprocess.call(('open', p ))			
		elif platform.system() == 'Windows':    # Windows
			AppLog.log( LOG_INFO , "Ouverture ...")									
			os.startfile( p )
		else:                                   # linux variants
			AppLog.log( LOG_INFO , "Ouverture ...")									
			subprocess.call(('xdg-open', p ))

	def onOpendirFile( self ):
		
		p = str(Path(self.filename).resolve())
		
		if platform.system() == 'Darwin':       # macOS
			AppLog.log( LOG_INFO , "Ouverture dossier ... ", p)					
			os.system( self.fileExplorer + ' "' + p + '"')
		elif platform.system() == 'Windows':    # Windows
			p = os.path.dirname(p)
			AppLog.log( LOG_INFO , "Ouverture dossier ... ", p)					
			os.system( self.fileExplorer + ' "' + p + '"')
		else:                                   # linux variants
			AppLog.log( LOG_INFO , "Ouverture dossier ... ", p)					
			os.system( self.fileExplorer + ' "' + p + '"')
			
		
	def onCopyPhotoName(self) :
		
		p = str(Path(self.filename).resolve())
		QApplication.clipboard().setText(p)
				
	def getFileTypeIcon (self, filename ) :
		
		if filename is None :
			return '_NONE_' 
		
		if filename[0:4] == 'http' :
			return 'http'
		
		file_name, file_extension = os.path.splitext(filename)		
		file_extension = file_extension[1:] # remove prefix dot
		
		# .flac, .mp3, .ogg
		if file_extension in self.mediaTypesAudioList :		
			return 'audio'
		
		# 'mp4' ,xvid 
		if file_extension in self.mediaTypesVideoList :
			return 'video'

		# jpg, png
		if file_extension in self.mediaTypesPhotoList :
			return 'photo'

		# pdf
		if file_extension in self.mediaTypesDocList :
			return 'pdf'

		return 'default'
		

	def onToFile(self) :
		self.setModified(False)

	def onMediaNameEditChanged( self) :

		if self.filename is not None and self.mediaName.text() != os.path.basename( self.filename ) :
			mediaNewFileName = os.path.join( os.path.dirname(self.filename), self.mediaName.text())
			if os.path.exists( mediaNewFileName ) :
				self.mediaName.setStyleSheet("color: red;")
			else :
				self.mediaName.setStyleSheet("color: green;")
				self.setModified(True)

	def onMediaNameEditValid( self ) :

		if self.filename is not None and self.mediaName.text() != os.path.basename( self.filename ) :
			mediaNewFileName = os.path.join( os.path.dirname(self.filename), self.mediaName.text())
			mediaCurrentName = self.filename
			if os.path.exists( mediaNewFileName ) :
				msg = QMessageBox()
				msg.setIcon(QMessageBox.Critical)
				msg.setText("Ce fichier existe déjà.")
				msg.setInformativeText(mediaNewFileName)
				msg.setWindowTitle("Error")
				x = msg.exec_()
			else :
				ret = QMessageBox.question(self, 'Confirmez', "Confirmez vous le changement de nom du média ?", QMessageBox.Yes | QMessageBox.Cancel, QMessageBox.Cancel)
				if ret == QMessageBox.Yes:

					try:
						import shutil
						shutil.move( mediaCurrentName, mediaNewFileName)
						if os.path.exists( mediaCurrentName + ".shp" ) :
							shutil.move( mediaCurrentName + ".shp", mediaNewFileName + ".shp")
						self.filenamelist[self.filenamelistIndex] = mediaNewFileName
						self.filename  = mediaNewFileName
						self.setModified(False)
						self.mediaName.setStyleSheet("color: black;")

					except Exception as e:
						self.printDebug(e.errno)
						self.printDebug(e)
						msg = QMessageBox()
						msg.setIcon(QMessageBox.Critical)
						msg.setText("Error")
						msg.setInformativeText(e)
						msg.setWindowTitle("Error")
						x = msg.exec_()
		else:
			self.mediaName.setStyleSheet("color: black;")


	def loadDataList( self, filenamelist ) :

		self.filenamelistIndex = 0

		self.filenamelist = filenamelist

		# Add rootMedia to each media	
		self.printDebug( "loadDatalist:" )
		self.printDebug( "Prefix w.rootmedia ? : ",self.rootMedia )
		
		if filenamelist is not None and self.rootMedia != '' :
			index=0
			for fileName in filenamelist :
				
				if len(fileName) >=1 and fileName[0] == "/" : 
					fileName = fileName[1:]
				
				if fileName.find("http://") == 0 or fileName.find("https://") == 0  :
					self.printDebug( "no (http link)" )
					filenamelist[index] = fileName
				else :				
					if fileName.find(self.rootMedia) == -1 :
						self.printDebug( "not found : so add to : ",fileName )
						filenamelist[index] = os.path.join(self.rootMedia, fileName)					
						self.printDebug( "result = ",filenamelist[index] )
					if fileName.find(self.rootMedia) == 0 :
						self.printDebug( "found @0 : ",fileName )					
					if fileName.find(self.rootMedia) > 0 :
						self.printDebug( "found >0 : ",fileName )					
					index=index+1
		else :			
			self.printDebug( "filenamelist is null (loadDataList)" )

		self.printDebug( "Load media : ", filenamelist[self.filenamelistIndex]  )
		
		if fileName.find("http://") == 0 or fileName.find("https://") == 0  :		
			pass
		else :
			if not os.path.isfile(filenamelist[self.filenamelistIndex]) :
				self.printDebug( "ERR : cant access media." )
			else :	
				# Load actual data
				self.loadFileData( filenamelist[self.filenamelistIndex] )
		
		
	def loadFileData( self, filename ) :

		self.filename = filename

		self.printDebug("loadFileData, media is: " , self.filename )

		self.setWindowTitle(self.filename)

		if self.filename is None :
			self.mediaName.setText( "" )

		self.displayMediaMetaDatas( {} ) 
		
		fileType = self.getFileTypeIcon(self.filename)
		if fileType== 'http' :
			self.mediaName.setText( self.filename )
			
			mediaMetaObj = MediaTypeHttp( self.filename, tempDir = self.tempdir )
			
			self.viewer.setPhotoFromPixmap( mediaMetaObj.getPixmap() ) 
			self.viewer.setZoomZero()
			self.displayMediaMetaDatas( mediaMetaObj.getMetaInformationList()  )  
		
		if fileType == 'audio' :
			self.mediaName.setText( os.path.basename(self.filename) )

			mediaMetaObj = MediaTypeAudio( self.filename, self.filename, tempDir = self.tempdir )
			
			self.viewer.setPhotoFromPixmap( mediaMetaObj.getPixmap() ) 
			self.viewer.setZoomZero()
			self.displayMediaMetaDatas( mediaMetaObj.getMetaInformationList()  ) 

		if fileType == 'pdf' :
			self.mediaName.setText( os.path.basename(self.filename) )

			mediaMetaObj = MediaTypePdf( self.filename, self.filename, tempDir = self.tempdir )
			
			self.viewer.setPhotoFromPixmap( mediaMetaObj.getPixmap() ) 
			self.viewer.setZoomFit()
			self.displayMediaMetaDatas( mediaMetaObj.getMetaInformationList()  ) 

		if fileType == 'video' :
			self.mediaName.setText( os.path.basename(self.filename) )

			mediaMetaObj = MediaTypeVideo( self.filename, self.filename, tempDir = self.tempdir )
			
			self.viewer.setPhotoFromPixmap( mediaMetaObj.getPixmap() ) 
			self.viewer.setZoomFit()						
			self.displayMediaMetaDatas( mediaMetaObj.getMetaInformationList()  ) 
			
		if fileType == 'photo' :
			self.mediaName.setText( os.path.basename(self.filename) )

			mediaMetaObj = MediaTypePhoto( self.filename, self.filename, tempDir = self.tempdir)
			
			self.viewer.setPhotoFromPixmap( mediaMetaObj.getPixmap() ) 
			self.viewer.setZoomFit()			
			self.displayMediaMetaDatas( mediaMetaObj.getMetaInformationList()  ) 
			
		#print( "FILE TYPE : ", self.getFileTypeIcon( self.filename ) )
		if fileType != "photo" :
			self.buttonEditFile.setEnabled(False)			
			
		self.currentItem = None
		self.setModified(False)
		self.mediaName.setStyleSheet("color: black;")

	def displayMediaMetaDatas( self , rowDatas ) :
		
		self.mediaInfoText.setRowCount(0)

		rowNumber = 0
		
		for key, metaLine in rowDatas.items():
			
			self.mediaInfoText.insertRow(self.mediaInfoText.rowCount()) 
			libItem = QTableWidgetItem( metaLine['lib'])
			libItem.setToolTip(metaLine['tooltip'])
			self.mediaInfoText.setItem(rowNumber , 0, libItem)			
			self.mediaInfoText.setItem(rowNumber , 1, QTableWidgetItem( str(metaLine['value']) ))
			
			rowNumber = rowNumber + 1

		header = self.mediaInfoText.horizontalHeader()
		header.setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)       
		header.setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)

	def getValue(self):

		if self.Reset is False :
			if self.Accept is False :
				return (None)
			else :
							
				# Remove rootMedia to each photo				
				self.printDebug("rootMedia = ", self.rootMedia)
				if self.filenamelist is not None and self.rootMedia != '' :
					index=0
					for fileName in self.filenamelist :
						
						if fileName == '' : continue
						
						rootMediaPos = fileName.find(self.rootMedia)
						if rootMediaPos == -1 :
							self.printDebug("NO rootmedia : ", fileName)
						if rootMediaPos == 0 :							
							self.printDebug("rootmedia @0 : ", fileName)
							self.filenamelist[index] = self.filenamelist[index].replace( self.rootMedia, '')						
						if rootMediaPos > 0 :
							self.printDebug("rootmedia >0 : ", rootMediaPos)
							self.filenamelist[index]  = self.filenamelist[index][rootMediaPos + len(self.rootMedia):]
						index=index+1
				else :
					self.printDebug("filenamelist is null (loadDataList)")				
					
				return (self.filenamelist)
		else :
			return ( [""] )
			
