#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

        Table view widget

        07/08/2021 11:03

        lvardon@laposte.net - 2021

        Licence libre

'''
import os, re, sys, shutil, io, csv

from PyQt5.QtCore import QByteArray,QSettings, QRect,Qt, pyqtSignal, pyqtSlot, QModelIndex
from PyQt5.QtWidgets import QApplication, QTableView, QHeaderView, QAction, QMenu, QToolTip, QStyledItemDelegate,QStyle            
from PyQt5.QtGui import QFont, QIcon, QPixmap , QImage, QColor, QPalette, QKeySequence

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

import lib.setting as AppSetting

class TableView(QTableView):

	cellClickSignal =  pyqtSignal( int )
	cellDoubleClickSignal =  pyqtSignal( int )
	
	parent = None
	log = None
	restoreState = True
	sourceOrigin = None

	def __init__(self, parent=None, viewname="", sourceOrigin="" ):
		super().__init__(parent)
		
		self.parent = parent
		self.setupUi()
		
		self.viewname = viewname
	
		if sourceOrigin != "" :
			self.sourceOrigin = sourceOrigin
		else :
			self.sourceOrigin = "tableview"
		
		self.clicked.connect(self.onViewClicked)
		self.doubleClicked.connect(self.onRowDoubleClick)
		
		self.setSelectionBehavior(QTableView.SelectRows)
		self.setSelectionMode(QTableView.SingleSelection)

		
	def setupUi( self ) :
				
		AppLog.log( LOG_INFO , "setupUi()")	
		
		self.setAlternatingRowColors(True)
		self.setStyleSheet("alternate-background-color: rgb(240, 240, 240)")

		self.setShowGrid(False)
		self.hView = TableViewHeader(self)
		self.setHorizontalHeader(self.hView)
		self.verticalHeader().hide()
		
		self.hView.setSortIndicatorShown(True)

		self.horizontalHeader().setStretchLastSection(True)
		self.resizeColumnsToContents()

		self.setSortingEnabled(True)
		
		self.horizontalHeader().setSectionsMovable(True)		
		
	def storeState( self ) :
		
		AppLog.log( LOG_INFO , "Store window state: "+self.sourceOrigin + "/"+self.viewname+"_geometry" )	
		AppSetting.set(self.sourceOrigin + "/"+self.viewname+"_geometry", self.hView.saveState())
	
	def loadState( self ) :

		AppLog.log( LOG_INFO , "Load window state: "+ self.sourceOrigin + "/"+self.viewname+"_geometry" )	
		tableViewState = AppSetting.get(self.sourceOrigin + "/"+self.viewname+"_geometry", "")	
		if type(tableViewState) is QByteArray:		
			self.hView.restoreState(AppSetting.get(self.sourceOrigin + "/"+self.viewname+"_geometry", ""))

	def showEvent( self, event) :
		
		AppLog.log( LOG_INFO , "showEvent()")	
		self.loadState()
				
	def hideEvent( self, event) :
		
		AppLog.log( LOG_INFO , "hideEvent()")	
		self.storeState()

	@pyqtSlot(QModelIndex)
	def onViewClicked(self, index):

		row = index.row()
		column = index.column()
		model = self.model()		
		rowId = model.index(index.row(),0).data()

		self.cellClickSignal.emit(rowId)
		
	@pyqtSlot(QModelIndex)
	def onRowDoubleClick(self, index):
		
		row = index.row()
		column = index.column()
		model = self.model()		
		rowId = model.index(index.row(),0).data()
		AppLog.log( LOG_INFO , "TableView : Cell double click : rowId = " + str(rowId) )
		
		self.cellDoubleClickSignal.emit(rowId)
		
	def keyPressEvent(self, event):	
		
		if event.matches(QKeySequence.Copy):
			self.copySelection()
			        
	def copySelection(self):
		
		selection = self.selectedIndexes()
		if selection:
			rows = sorted(index.row() for index in selection)
			columns = sorted(index.column() for index in selection)
			rowcount = rows[-1] - rows[0] + 1
			colcount = columns[-1] - columns[0] + 1
			table = [[''] * colcount for _ in range(rowcount)]
			for index in selection:
				row = index.row() - rows[0]
				column = index.column() - columns[0]
				Celldata = index.model().data(index, Qt.UserRole)					
				table[row][column] = Celldata				
			stream = io.StringIO()
			csv.writer(stream, delimiter='\t').writerows(table)
			clipboard = QApplication.clipboard()            
			clipboard.setText(stream.getvalue()) 
						
class TableViewHeader(QHeaderView):
	
	def __init__(self, parent):
		
		QHeaderView.__init__(self, Qt.Horizontal, parent)
		
		self.parent = parent
		self.setupUi()
		
		self.setContextMenuPolicy(Qt.CustomContextMenu)
		self.customContextMenuRequested.connect(self.contextMenuEvent)
		
	def setupUi( self ) :
		
		self.setStyleSheet("::section{background-color: #bfcee0; color: black; }")

		self.setSectionsClickable(True)

	def contextMenuEvent(self, pos):

		logicalColIndex = self.logicalIndexAt(pos)	
		visualColIndex = self.visualIndex(logicalColIndex)	 
		model = self.model()
		maxColCount = model.columnCount()
		
		cmenu = QMenu(self)

		moveAtStartAction = cmenu.addAction( "En première position", lambda: self.onMoveAtStart(visualColIndex))
		moveAtEndAction = cmenu.addAction( "En dernière position", lambda: self.onMoveAtEnd(visualColIndex, maxColCount ))

		cmenu.addSeparator()
					
		cmenu.exec_(self.viewport().mapToGlobal(pos))

	def onMoveAtStart( self, visualColumnIndex ) :		
		self.moveSection(visualColumnIndex, 0)

	def onMoveAtEnd( self, visualColumnIndex, maxColCount) :
		self.moveSection(visualColumnIndex, maxColCount-1)

		
class TableCellImageDelegate(QStyledItemDelegate):
	
	def __init__(self, parent, tableName, iconList, rootMedia, icoW=None, icoH=None ):
				
		QStyledItemDelegate.__init__(self, parent)		
		self.iconList = iconList
		self.rootMedia = rootMedia
		self.iconPath = ""
		self.tableName = tableName
		
		if icoW is None or icoH is None :
			
			iniConfigFile = "config/app.ini"
			settings=QSettings(iniConfigFile, QSettings.IniFormat)

			if settings.value('tableview/rowIconW') is not None:			
				icoW = settings.value('tableview/rowIconW')					

			if settings.value('tableview/rowIconH') is not None:			
				icoH = settings.value('tableview/rowIconH')						

		self.icoW = int(icoW)
		self.icoH = int(icoH)
                
	def paint(self, painter, option, index):
		
		super().paint(painter, option, index)		
				
		Celldata = index.model().data(index, Qt.UserRole)		
								
		if Celldata != "" and Celldata is not None :	
				
			QApplication.setOverrideCursor(Qt.WaitCursor)				
			try :

				if self.iconList is None :

					self.iconPath = os.path.join(self.rootMedia, Celldata)
					
					if os.path.exists(self.iconPath) is False :						
						AppLog.log( LOG_WARNING , "Média non trouvé: "+ self.iconPath )	
						self.iconPath = "img/dataicons/notfound.png"	# FIX			
				else :
					self.iconPath = self.iconList[str(Celldata)]
					
				file_name, file_extension = os.path.splitext(self.iconPath)
				file_extension = file_extension[1:] # remove prefix dot					

				if file_extension.lower() in ('pdf', 'docx', 'txt') :					
					self.iconPath = "libwidget/mediastype/icons/document.png"

				if file_extension.lower() in ('mp3', 'flac') :					
					self.iconPath = "libwidget/mediastype/icons/audio.png"

				if file_extension.lower() in ('mp4') :					
					self.iconPath = "libwidget/mediastype/icons/video.png"
				
				import hashlib 
				pixmapUniq = self.tableName + self.iconPath
				result = hashlib.md5( pixmapUniq.encode())   
				md5sig = result.hexdigest()

				iniConfigFile = "config/app.ini"
				settings=QSettings(iniConfigFile, QSettings.IniFormat)
				tempdir = "temp"
				if settings.value('app/tempDir') is not None:			
					tempdir = settings.value('app/tempDir')						

				pixmapFile = str(tempdir) + "/" + md5sig + ".pixmap"

				if os.path.isfile( pixmapFile ) is False :
				
					image = QImage(str(self.iconPath))
					pixmap = QPixmap.fromImage(image)
					pixmap = pixmap.scaled(self.icoW, self.icoH, Qt.KeepAspectRatio)

					pixmap.save(pixmapFile, "PNG")
				
				else :
					pixmap = QPixmap()
					pixmap.load(pixmapFile)
					
				# Centrer l'image
				x = option.rect.center().x() - pixmap.rect().width() / 2
				y = option.rect.center().y() - pixmap.rect().height() / 2

				'''
				from PyQt5.QtGui import QBrush, QPalette
				from PyQt5.QtWidgets import QStyle
				
				if option.state & QStyle.State_Selected:
					option.palette.setColor( QPalette.HighlightedText, option.palette.highlight().color())	
				else :
					option.palette.setColor( QPalette.Text, option.palette.color(QPalette.Background)  )
				'''			
				       				
				painter.drawPixmap(QRect(x, y, pixmap.rect().width(), pixmap.rect().height()), pixmap)				
				
			except Exception as e:
				AppLog.log( LOG_ERROR , str(e) )	
					
		QApplication.restoreOverrideCursor()					

					
class TableCellItemDelegate(QStyledItemDelegate):
	
	def initStyleOption(self, option, index):
		
		super(TableCellItemDelegate, self).initStyleOption(option, index)

		color = QColor("#FFFFFF")

		if index.column() == 1:
			color = QColor("#34ebc6")
		elif index.column() == 2:
			color = QColor("#FFFFFF")
		elif index.column() == 3:
			color = QColor("#9546c7")
		elif index.column() == 6:
			color = QColor("#9546c7")
			
		cg = (
			QPalette.Normal
			if option.state & QStyle.State_Enabled
			else QPalette.Disabled
		)
		if option.state & QStyle.State_Selected:
			option.palette.setColor(cg, QPalette.HighlightedText, color)

		option.palette.setBrush(QPalette.Text, color)
